import pytz
from django.utils import timezone, translation


# Change language
class Language:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.user.is_authenticated:
            if request.user.profile.language:
                translation.activate(request.user.profile.language)

        response = self.get_response(request)
        return response


# Change timezone
class Timezone:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if request.user.is_authenticated:
            if request.user.profile.geobase and request.user.profile.geobase.timezone.name:
                timezone.activate(pytz.timezone(request.user.profile.geobase.timezone.name))
            else:
                timezone.deactivate()

        response = self.get_response(request)
        return response



# Generated by Django 2.2 on 2019-05-06 20:16

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_tasker_account.models
import django_tasker_account.validators


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Geobase',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('latitude', models.FloatField(verbose_name='Latitude')),
                ('longitude', models.FloatField(verbose_name='Longitude')),
            ],
            options={
                'verbose_name': 'Geobase',
                'verbose_name_plural': 'Geobase',
            },
        ),
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language', models.CharField(choices=[('af', 'Afrikaans'), ('ar', 'Arabic'), ('ast', 'Asturian'), ('az', 'Azerbaijani'), ('bg', 'Bulgarian'), ('be', 'Belarusian'), ('bn', 'Bengali'), ('br', 'Breton'), ('bs', 'Bosnian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('cy', 'Welsh'), ('da', 'Danish'), ('de', 'German'), ('dsb', 'Lower Sorbian'), ('el', 'Greek'), ('en', 'English'), ('en-au', 'Australian English'), ('en-gb', 'British English'), ('eo', 'Esperanto'), ('es', 'Spanish'), ('es-ar', 'Argentinian Spanish'), ('es-co', 'Colombian Spanish'), ('es-mx', 'Mexican Spanish'), ('es-ni', 'Nicaraguan Spanish'), ('es-ve', 'Venezuelan Spanish'), ('et', 'Estonian'), ('eu', 'Basque'), ('fa', 'Persian'), ('fi', 'Finnish'), ('fr', 'French'), ('fy', 'Frisian'), ('ga', 'Irish'), ('gd', 'Scottish Gaelic'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hsb', 'Upper Sorbian'), ('hu', 'Hungarian'), ('hy', 'Armenian'), ('ia', 'Interlingua'), ('id', 'Indonesian'), ('io', 'Ido'), ('is', 'Icelandic'), ('it', 'Italian'), ('ja', 'Japanese'), ('ka', 'Georgian'), ('kab', 'Kabyle'), ('kk', 'Kazakh'), ('km', 'Khmer'), ('kn', 'Kannada'), ('ko', 'Korean'), ('lb', 'Luxembourgish'), ('lt', 'Lithuanian'), ('lv', 'Latvian'), ('mk', 'Macedonian'), ('ml', 'Malayalam'), ('mn', 'Mongolian'), ('mr', 'Marathi'), ('my', 'Burmese'), ('nb', 'Norwegian Bokmål'), ('ne', 'Nepali'), ('nl', 'Dutch'), ('nn', 'Norwegian Nynorsk'), ('os', 'Ossetic'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pt', 'Portuguese'), ('pt-br', 'Brazilian Portuguese'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sq', 'Albanian'), ('sr', 'Serbian'), ('sr-latn', 'Serbian Latin'), ('sv', 'Swedish'), ('sw', 'Swahili'), ('ta', 'Tamil'), ('te', 'Telugu'), ('th', 'Thai'), ('tr', 'Turkish'), ('tt', 'Tatar'), ('udm', 'Udmurt'), ('uk', 'Ukrainian'), ('ur', 'Urdu'), ('vi', 'Vietnamese'), ('zh-hans', 'Simplified Chinese'), ('zh-hant', 'Traditional Chinese')], default='en', max_length=5, verbose_name='Language')),
                ('gender', models.SmallIntegerField(blank=True, choices=[(1, 'Male'), (2, 'Female')], null=True, verbose_name='Gender')),
                ('birth_date', models.DateField(blank=True, null=True, verbose_name='Birth date')),
                ('phone', models.BigIntegerField(blank=True, null=True, validators=[django_tasker_account.validators.mobile_number])),
                ('avatar', models.ImageField(blank=True, null=True, upload_to=django_tasker_account.models.Profile.path)),
                ('geobase', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='django_tasker_account.Geobase', verbose_name='Geobase')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
        ),
        migrations.CreateModel(
            name='GeobaseTimezone',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Time zone')),
            ],
            options={
                'verbose_name': 'Geobase timezone',
                'verbose_name_plural': 'Geobase timezone',
                'unique_together': {('name',)},
            },
        ),
        migrations.CreateModel(
            name='GeobaseProvince',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ru', models.CharField(db_index=True, max_length=255, verbose_name='Name russian')),
                ('en', models.CharField(db_index=True, max_length=255, verbose_name='Name english')),
            ],
            options={
                'verbose_name': 'Geobase province',
                'verbose_name_plural': 'Geobase province',
                'unique_together': {('ru', 'en')},
            },
        ),
        migrations.CreateModel(
            name='GeobaseLocality',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ru', models.CharField(db_index=True, max_length=255, verbose_name='Name russian')),
                ('en', models.CharField(db_index=True, max_length=255, verbose_name='Name english')),
            ],
            options={
                'verbose_name': 'Geobase locality',
                'verbose_name_plural': 'Geobase locality',
                'unique_together': {('ru', 'en')},
            },
        ),
        migrations.CreateModel(
            name='GeobaseCountry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ru', models.CharField(db_index=True, max_length=255, verbose_name='Name russian')),
                ('en', models.CharField(db_index=True, max_length=255, verbose_name='Name english')),
            ],
            options={
                'verbose_name': 'Geobase сountry',
                'verbose_name_plural': 'Geobase сountry',
                'unique_together': {('ru', 'en')},
            },
        ),
        migrations.AddField(
            model_name='geobase',
            name='country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_tasker_account.GeobaseCountry', verbose_name='Country'),
        ),
        migrations.AddField(
            model_name='geobase',
            name='locality',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_tasker_account.GeobaseLocality', verbose_name='Locality'),
        ),
        migrations.AddField(
            model_name='geobase',
            name='province',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='django_tasker_account.GeobaseProvince', verbose_name='Province'),
        ),
        migrations.AddField(
            model_name='geobase',
            name='timezone',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_tasker_account.GeobaseTimezone', verbose_name='Time zone'),
        ),
        migrations.CreateModel(
            name='Oauth',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('oauth_id', models.CharField(max_length=255, verbose_name='Oauth ID')),
                ('provider', models.IntegerField(choices=[(1, 'Google'), (2, 'Yandex'), (3, 'Mail.ru'), (4, 'VK.com'), (5, 'Facebook')], verbose_name='Server')),
                ('access_token', models.CharField(max_length=255, verbose_name='Access token')),
                ('refresh_token', models.CharField(blank=True, max_length=255, null=True, verbose_name='Refresh token')),
                ('expires_in', models.DateTimeField(verbose_name='Expires date')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'OAuth',
                'verbose_name_plural': 'OAuth',
                'unique_together': {('oauth_id', 'provider')},
            },
        ),
        migrations.AlterUniqueTogether(
            name='geobase',
            unique_together={('country', 'province', 'locality')},
        ),
    ]



# Generated by Django 2.2 on 2019-05-06 20:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_tasker_account', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='oauth',
            name='refresh_token',
        ),
    ]



import hashlib
import json
import os

import requests
import logging

from ipaddress import ip_address
from timezonefinder import TimezoneFinder

from django.conf import settings
from django.shortcuts import get_object_or_404
from django.contrib.gis.geoip2 import GeoIP2
from django.core.cache import cache
from django.core.handlers.wsgi import WSGIRequest

from . import models

logger = logging.getLogger('django_tasker_account')


def detect_ip(query: WSGIRequest) -> models.Geobase:
    """
    Calculates geolocation by IP address.

    :param query: ip address IPv4 or IPv6.
    :returns: Geobase
    """
    if query and isinstance(query, WSGIRequest):
        x_forwarded_for = query.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = query.META.get('REMOTE_ADDR')
    else:
        ip = query

    try:
        ip = ip_address(ip)
    except ValueError:
        geobase = _geocoder("{longitude},{latitude}".format(latitude="51.507351", longitude="-0.12766"))
        return geobase

    # Cache IP address
    cache_detect = cache.get("{name}-{ip}".format(ip=str(ip), name=__name__))
    if cache_detect:
        logger.debug("Get geobase cached data ip:{ip}, id:{id}".format(ip=str(ip), id=cache_detect))
        return models.Geobase.objects.get(id=cache_detect)

    latitude = None
    longitude = None

    YANDEX_LOCATOR_KEY = getattr(settings, 'YANDEX_LOCATOR_KEY', os.environ.get('YANDEX_LOCATOR_KEY'))
    if ip.version == 4 and ip.is_global and YANDEX_LOCATOR_KEY:
        json_params = json.dumps({
            "common": {
                "version": "1.0",
                "api_key": YANDEX_LOCATOR_KEY,
            },
            "gsm_cells": [],
            "wifi_networks": [],
            "ip": {
                "address_v4": str(ip)
            }
        })
        response = requests.post('https://api.lbs.yandex.net/geolocation', params={'json': json_params})
        locator_result = response.json()
        if response.status_code == 200:
            if locator_result.get('position').get('latitude') and locator_result.get('position').get('longitude'):
                latitude = locator_result.get('position').get('latitude')
                longitude = locator_result.get('position').get('longitude')

    elif ip.version == 4 and ip.is_global:
        g = GeoIP2()
        geoip = g.city(str(ip))
        latitude = geoip.get('latitude')
        longitude = geoip.get('longitude')
    elif ip.version == 6 and ip.is_global:
        g = GeoIP2()
        geoip = g.city(str(ip))
        latitude = geoip.get('latitude')
        longitude = geoip.get('longitude')
    else:
        latitude = '51.507351'
        longitude = '-0.12766'

    # Cache IP address
    geobase = detect_geo("{longitude},{latitude}".format(latitude=latitude, longitude=longitude))
    cache.set("{name}-{ip}".format(ip=str(ip), name=__name__), geobase.id, 60 * 60 * 24 * 30)
    return geobase


def detect_geo(query: str) -> models.Geobase:
    """
    Calculates geolocation an address object.

    :param query: Address or geographical the object
    :returns: Geobase
    """
    cache_key = hashlib.sha256()
    cache_key.update(query.encode('utf-8'))

    cache_geocoder = cache.get("{name}-{hash}".format(hash=cache_key.hexdigest(), name=__name__))
    if cache_geocoder:
        return get_object_or_404(models.Geobase, id=cache_geocoder)

    en = _geocoder(query, language='en')
    if not en:
        en = {
            'country': 'United Kingdom',
            'province': 'England',
            'locality': 'London',
            'timezone': 'Europe/London',
            'latitude': 51.507351,
            'longitude': -0.127664,
        }

    query_geocoder = "{longitude},{latitude}".format(longitude=en.get('longitude'), latitude=en.get('latitude'))
    ru = _geocoder(query_geocoder, language='ru')

    if not ru:
        en = {
            'country': 'United Kingdom',
            'province': 'England',
            'locality': 'London',
            'timezone': 'Europe/London',
            'latitude': 51.507351,
            'longitude': -0.127664,
        }

        ru = {
            'country': 'Великобритания',
            'province': 'Англия',
            'locality': 'Лондон',
            'timezone': 'Europe/London',
            'latitude': 51.507351,
            'longitude': -0.127664,
        }

    country, created = models.GeobaseCountry.objects.update_or_create(
        en=en.get('country'),
        ru=ru.get('country')
    )
    if created:
        logger.debug("created new country object {en}, {ru}".format(en=en.get('country'), ru=ru.get('country')))

    province, created = models.GeobaseProvince.objects.update_or_create(
        en=en.get('province'),
        ru=ru.get('province')
    )
    if created:
        logger.debug("created new province object {en}, {ru}".format(en=en.get('province'), ru=ru.get('province')))

    locality, created = models.GeobaseLocality.objects.update_or_create(
        en=en.get('locality'),
        ru=ru.get('locality')
    )
    if created:
        logger.debug("created new locality object {en}, {ru}".format(en=en.get('locality'), ru=ru.get('locality')))

    timezone, created = models.GeobaseTimezone.objects.update_or_create(name=en.get('timezone'))
    if created:
        logger.debug("created new timezone object {name}".format(name=en.get('timezone')))

    geobase, created = models.Geobase.objects.update_or_create(
        country=country,
        province=province,
        locality=locality,
        defaults={'timezone': timezone, 'longitude': en.get('longitude'), 'latitude': en.get('latitude')}
    )
    if created:
        logger.debug("created new geo object {country}, {province}, {locality}".format(
            country=country,
            province=province,
            locality=locality
        ))

    cache.set("{name}-{hash}".format(hash=cache_key.hexdigest(), name=__name__), geobase.id, 60 * 60 * 24 * 30)
    return geobase


def _geocoder(query, language='en'):
    data = {
        'country': 'N/A',
        'province': None,
        'locality': None,
        'timezone': 'UTC',
        'latitude': 0,
        'longitude': 0,
    }

    params = {
        "apikey": getattr(settings, 'YANDEX_MAP_KEY', os.environ.get('YANDEX_MAP_KEY')),
        "format": "json",
        "geocode": query,
        "kind": "locality",
    }

    if language == 'ru':
        params['lang'] = 'ru_RU'
    else:
        params['lang'] = 'en_US'

    response = requests.get('https://geocode-maps.yandex.ru/1.x/', params=params)
    json_yandex_map = response.json()
    if 'error' in json_yandex_map:
        return data

    tf = TimezoneFinder()
    geoobject = json_yandex_map.get('response').get('GeoObjectCollection').get('featureMember')
    for item in geoobject:
        item = item.get('GeoObject')

        if 'Point' in item:
            pos = item.get('Point').get('pos')
            pos = pos.split(" ")

        components = item.get('metaDataProperty').get('GeocoderMetaData').get('Address').get('Components')
        for component in components:
            if 'kind' in component and component.get('kind') == 'country':
                data['country'] = component.get('name')
            elif 'kind' in component and component.get('kind') == 'province':
                data['province'] = component.get('name')
            elif 'kind' in component and component.get('kind') == 'locality':
                data['locality'] = component.get('name')

        if not data.get('locality'):
            data['locality'] = data.get('province')

        data['longitude'] = float(pos[0])
        data['latitude'] = float(pos[1])
        data['timezone'] = tf.timezone_at(lng=data.get('longitude'), lat=data.get('latitude'))
        return data



import os

from django.test import TestCase, override_settings
from django_tasker_account import geobase


@override_settings(
    ALLOWED_HOSTS=['localhost'],
    CACHES={'default': {'BACKEND': 'django.core.cache.backends.dummy.DummyCache'}}
)
class Geobase(TestCase):

    def test_geocoder(self):
        result = geobase.detect_geo(query='Москва')
        self.assertEqual(result.country.en, 'Russia')
        self.assertEqual(result.province.en, 'Moscow')
        self.assertEqual(result.locality.en, 'Moscow')
        self.assertEqual(result.timezone.name, 'Europe/Moscow')
        self.assertEqual(result.latitude, 55.753215)
        self.assertEqual(result.longitude, 37.622504)

    def test_detect_ip(self):
        result = geobase.detect_ip(query='8.8.8.8')
        self.assertEqual(result.country.en, 'United States of America')
        self.assertEqual(result.province.en, 'District of Columbia')
        self.assertEqual(result.locality.en, 'City of Washington')
        self.assertEqual(result.timezone.name, 'America/New_York')
        self.assertEqual(result.latitude, 38.899513)
        self.assertEqual(result.longitude, -77.036527)

        if not os.environ.get('TRAVIS'):
            result = geobase.detect_ip(query='2a02:6b8::feed:0ff')
            self.assertEqual(result.country.en, 'Russia')
            self.assertEqual(result.province.en, 'Moscow')
            self.assertEqual(result.locality.en, 'Moscow')
            self.assertEqual(result.timezone.name, 'Europe/Moscow')
            self.assertEqual(result.latitude, 55.755814)
            self.assertEqual(result.longitude, 37.617635)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DjangoTaskerAccountConfig(AppConfig):
    name = 'django_tasker_account'
    verbose_name = _("Django tasker account")
